@@ -53,7 +53,7 @@
 
 
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -490,10 +490,10 @@ gif_lzw_fill_buffer (GifContext *context)
 
 	if (context->code_done) {
 		if (context->code_curbit >= context->code_lastbit) {
-                        g_set_error (context->error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                                     _("GIF file was missing some data (perhaps it was truncated somehow?)"));
+                        g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("GIF file was missing some data (perhaps it was truncated somehow?)"));
 
 			return -2;
 		}
@@ -573,10 +573,10 @@ gif_lzw_clear_code (GifContext *context)
 #define CHECK_LZW_SP() G_STMT_START {                                           \
         if ((guchar *)context->lzw_sp >=                                        \
             (guchar *)context->lzw_stack + sizeof (context->lzw_stack)) {       \
-                 g_set_error (context->error,                                   \
-                             GDK_PIXBUF_ERROR,                                  \
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,                    \
-                             _("Stack overflow"));                              \
+                 g_set_error_literal (context->error,                           \
+                                      GDK_PIXBUF_ERROR,                         \
+                                      GDK_PIXBUF_ERROR_CORRUPT_IMAGE,           \
+                                      _("Stack overflow"));                     \
                 return -2;                                                      \
         }                                                                       \
 } G_STMT_END
@@ -633,10 +633,10 @@ lzw_read_byte (GifContext *context)
 			unsigned char buf[260];
 
                         /*  FIXME - we should handle this case */
-                        g_set_error (context->error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_FAILED,
-                                     _("GIF image loader cannot understand this image."));
+                        g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_FAILED,
+                                             _("GIF image loader cannot understand this image."));
                         return -2;
                         
 			if (ZeroDataBlock) {
@@ -662,20 +662,20 @@ lzw_read_byte (GifContext *context)
 
 		while (code >= context->lzw_clear_code) {
                         if (code >= (1 << MAX_LZW_BITS)) {
-                                g_set_error (context->error,
-                                             GDK_PIXBUF_ERROR,
-                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                                             _("Bad code encountered"));
+                                g_set_error_literal (context->error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                                     _("Bad code encountered"));
 				return -2;
                         }
                         CHECK_LZW_SP ();
 			*(context->lzw_sp)++ = context->lzw_table[1][code];
 
 			if (code == context->lzw_table[0][code]) {
-                                g_set_error (context->error,
-                                             GDK_PIXBUF_ERROR,
-                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                                             _("Circular table entry in GIF file"));
+                                g_set_error_literal (context->error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                                     _("Circular table entry in GIF file"));
 				return -2;
 			}
 			code = context->lzw_table[0][code];
@@ -860,10 +860,10 @@ gif_get_lzw (GifContext *context)
                                                 context->frame_height);
                 if (!context->frame->pixbuf) {
                         g_free (context->frame);
-                        g_set_error (context->error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                                     _("Not enough memory to load GIF file"));
+                        g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                             _("Not enough memory to load GIF file"));
                         return -2;
                 }
 
@@ -908,10 +908,10 @@ gif_get_lzw (GifContext *context)
 
                 /* Only call prepare_func for the first frame */
 		if (context->animation->frames->next == NULL) { 
-			if (context->animation->width == 0 )
-				context->animation->width = gdk_pixbuf_get_width(context->frame->pixbuf);
-			if (context->animation->height == 0)
-				context->animation->height = gdk_pixbuf_get_height (context->frame->pixbuf);
+                        if (context->animation->width == 0 )
+                                context->animation->width = gdk_pixbuf_get_width(context->frame->pixbuf);
+                        if (context->animation->height == 0)
+                                context->animation->height = gdk_pixbuf_get_height (context->frame->pixbuf);
 
                         if (context->prepare_func)
                                 (* context->prepare_func) (context->frame->pixbuf,
@@ -950,10 +950,10 @@ gif_get_lzw (GifContext *context)
                                 g_list_free (context->animation->frames);
                                 context->animation->frames = NULL;
                                 
-                                g_set_error (context->error,
-                                             GDK_PIXBUF_ERROR,
-                                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                                             _("Not enough memory to composite a frame in GIF file"));
+                                g_set_error_literal (context->error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                                     _("Not enough memory to composite a frame in GIF file"));
                                 return -2;
                         }
                     
@@ -1122,10 +1122,10 @@ gif_prepare_lzw (GifContext *context)
 	}
         
         if (context->lzw_set_code_size > MAX_LZW_BITS) {
-                g_set_error (context->error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("GIF image is corrupt (incorrect LZW compression)"));
+                g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("GIF image is corrupt (incorrect LZW compression)"));
                 return -2;
         }
 
@@ -1172,10 +1172,10 @@ gif_init (GifContext *context)
 
 	if (strncmp ((char *) buf, "GIF", 3) != 0) {
 		/* Not a GIF file */
-                g_set_error (context->error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("File does not appear to be a GIF file"));
+                g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("File does not appear to be a GIF file"));
 		return -2;
 	}
 
@@ -1293,10 +1293,10 @@ gif_get_frame_info (GifContext *context)
         if (!context->has_global_cmap) {
                 context->state = GIF_DONE;
                 
-                g_set_error (context->error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("GIF image has no global colormap, and a frame inside it has no local colormap."));
+                g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("GIF image has no global colormap, and a frame inside it has no local colormap."));
                 
 		return -2;
         }
@@ -1458,10 +1458,10 @@ gdk_pixbuf__gif_image_load (FILE *file, GError **error)
 	context = new_context ();
 
         if (context == NULL) {
-                g_set_error (error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                             _("Not enough memory to load GIF file"));
+                g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                     _("Not enough memory to load GIF file"));
                 return NULL;
         }
         
@@ -1471,10 +1471,10 @@ gdk_pixbuf__gif_image_load (FILE *file, GError **error)
 
 	if (gif_main_loop (context) == -1 || context->animation->frames == NULL) {
                 if (context->error && *(context->error) == NULL)
-                        g_set_error (context->error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                                     _("GIF file was missing some data (perhaps it was truncated somehow?)"));
+                        g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("GIF file was missing some data (perhaps it was truncated somehow?)"));
         }
         
         pixbuf = gdk_pixbuf_animation_get_static_image (GDK_PIXBUF_ANIMATION (context->animation));
@@ -1505,10 +1505,10 @@ gdk_pixbuf__gif_image_begin_load (GdkPixbufModuleSizeFunc size_func,
 	context = new_context ();
 
         if (context == NULL) {
-                g_set_error (error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                             _("Not enough memory to load GIF file"));
+                g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                     _("Not enough memory to load GIF file"));
                 return NULL;
         }
         
@@ -1526,11 +1526,11 @@ gdk_pixbuf__gif_image_stop_load (gpointer data, GError **error)
 	GifContext *context = (GifContext *) data;
         gboolean retval = TRUE;
         
-        if (context->state != GIF_DONE) {
-                g_set_error (error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("GIF image was truncated or incomplete."));
+        if (context->state != GIF_DONE || context->animation->frames == NULL) {
+                g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("GIF image was truncated or incomplete."));
 
                 retval = FALSE;
         }
@@ -1620,10 +1620,10 @@ gdk_pixbuf__gif_image_load_animation (FILE *file,
 	context = new_context ();
 
         if (context == NULL) {
-                g_set_error (error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                             _("Not enough memory to load GIF file"));
+                g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                     _("Not enough memory to load GIF file"));
                 return NULL;
         }
         
@@ -1632,10 +1632,10 @@ gdk_pixbuf__gif_image_load_animation (FILE *file,
 
 	if (gif_main_loop (context) == -1 || context->animation->frames == NULL) {
                 if (context->error && *(context->error) == NULL)
-                        g_set_error (context->error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                                     _("GIF file was missing some data (perhaps it was truncated somehow?)"));
+                        g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("GIF file was missing some data (perhaps it was truncated somehow?)"));
 
                 g_object_unref (context->animation);
                 context->animation = NULL;
@@ -1654,8 +1654,13 @@ gdk_pixbuf__gif_image_load_animation (FILE *file,
 	return animation;
 }
 
-void
-MODULE_ENTRY (gif, fill_vtable) (GdkPixbufModule *module)
+#ifndef INCLUDE_gif
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
+#else
+#define MODULE_ENTRY(function) void _gdk_pixbuf__gif_ ## function
+#endif
+
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 {
         module->load = gdk_pixbuf__gif_image_load;
         module->begin_load = gdk_pixbuf__gif_image_begin_load;
@@ -1664,8 +1669,7 @@ MODULE_ENTRY (gif, fill_vtable) (GdkPixbufModule *module)
         module->load_animation = gdk_pixbuf__gif_image_load_animation;
 }
 
-void
-MODULE_ENTRY (gif, fill_info) (GdkPixbufFormat *info)
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 {
         static GdkPixbufModulePattern signature[] = {
                 { "GIF8", NULL, 100 },
