@@ -367,9 +367,11 @@ attributes_start_element (GMarkupParseContext *context,
   guint i;
 
   if (strcmp (element_name, "property") == 0)
-    for (i = 0; names[i]; i++)
-      if (strcmp (names[i], "name") == 0)
-	parser_data->child_prop_name = g_strdup (values[i]);
+    {
+      for (i = 0; names[i]; i++)
+	if (strcmp (names[i], "name") == 0)
+	  parser_data->child_prop_name = g_strdup (values[i]);
+    }
   else if (strcmp (element_name, "packing") == 0)
     return;
   else
@@ -819,7 +821,6 @@ gtk_container_add_with_properties (GtkContainer *container,
   g_return_if_fail (GTK_IS_CONTAINER (container));
   g_return_if_fail (GTK_IS_WIDGET (widget));
   g_return_if_fail (widget->parent == NULL);
-  g_return_if_fail (widget->parent == container);
 
   g_object_ref (container);
   g_object_ref (widget);
@@ -1542,9 +1543,9 @@ gtk_container_foreach_full (GtkContainer       *container,
 /**
  * gtk_container_set_focus_child:
  * @container: a #GtkContainer
- * @widget: a #GtkWidget, or %NULL
+ * @child: a #GtkWidget, or %NULL
  *
- * Sets, or unsets if @widget is %NULL, the focused child of @container.
+ * Sets, or unsets if @child is %NULL, the focused child of @container.
  *
  * This function emits the GtkContainer::set_focus_child signal of
  * @container. Implementations of #GtkContainer can override the
@@ -1552,13 +1553,13 @@ gtk_container_foreach_full (GtkContainer       *container,
  */
 void
 gtk_container_set_focus_child (GtkContainer *container,
-			       GtkWidget    *widget)
+			       GtkWidget    *child)
 {
   g_return_if_fail (GTK_IS_CONTAINER (container));
-  if (widget)
-    g_return_if_fail (GTK_IS_WIDGET (widget));
+  if (child)
+    g_return_if_fail (GTK_IS_WIDGET (child));
 
-  g_signal_emit (container, container_signals[SET_FOCUS_CHILD], 0, widget);
+  g_signal_emit (container, container_signals[SET_FOCUS_CHILD], 0, child);
 }
 
 /**
