@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -43,9 +43,8 @@
 #include "gtkstock.h"
 #include "gtkwindow.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
-
 #include "gtkquartz.h"
+#include "gtkalias.h"
 #include "gdk/quartz/gdkquartz.h"
 
 typedef struct _GtkDragSourceSite GtkDragSourceSite;
@@ -144,6 +143,7 @@ struct _GtkDragFindData
 
   selection_data.selection = GDK_NONE;
   selection_data.data = NULL;
+  selection_data.length = -1;
   selection_data.target = _gtk_quartz_pasteboard_type_to_atom (type);
 
   if (gtk_target_list_find (info->target_list, 
@@ -156,7 +156,8 @@ struct _GtkDragFindData
 			     target_info,
 			     time);
 
-      _gtk_quartz_set_selection_data_for_pasteboard (sender, &selection_data);
+      if (selection_data.length >= 0)
+        _gtk_quartz_set_selection_data_for_pasteboard (sender, &selection_data);
       
       g_free (selection_data.data);
     }
@@ -182,12 +183,15 @@ gtk_drag_get_data (GtkWidget      *widget,
 		   GdkAtom         target,
 		   guint32         time)
 {
-  id <NSDraggingInfo> dragging_info = GDK_DRAG_CONTEXT_PRIVATE (context)->dragging_info;
-  NSPasteboard *pasteboard = [dragging_info draggingPasteboard];
+  id <NSDraggingInfo> dragging_info;
+  NSPasteboard *pasteboard;
   GtkSelectionData *selection_data;
   GtkDragDestInfo *info;
   GtkDragDestSite *site;
 
+  dragging_info = gdk_quartz_drag_context_get_dragging_info_libgtk_only (context);
+  pasteboard = [dragging_info draggingPasteboard];
+
   info = gtk_drag_get_dest_info (context, FALSE);
   site = g_object_get_data (G_OBJECT (widget), "gtk-drag-dest");
 
@@ -977,8 +981,8 @@ gtk_drag_dest_find_target (GtkWidget      *widget,
                            GdkDragContext *context,
                            GtkTargetList  *target_list)
 {
-  id <NSDraggingInfo> dragging_info = GDK_DRAG_CONTEXT_PRIVATE (context)->dragging_info;
-  NSPasteboard *pasteboard = [dragging_info draggingPasteboard];
+  id <NSDraggingInfo> dragging_info;
+  NSPasteboard *pasteboard;
   GtkWidget *source_widget;
   GList *tmp_target;
   GList *tmp_source = NULL;
@@ -988,6 +992,9 @@ gtk_drag_dest_find_target (GtkWidget      *widget,
   g_return_val_if_fail (GDK_IS_DRAG_CONTEXT (context), GDK_NONE);
   g_return_val_if_fail (!context->is_source, GDK_NONE);
 
+  dragging_info = gdk_quartz_drag_context_get_dragging_info_libgtk_only (context);
+  pasteboard = [dragging_info draggingPasteboard];
+
   source_widget = gtk_drag_get_source_widget (context);
 
   if (target_list == NULL)
@@ -1045,7 +1052,7 @@ gtk_drag_begin_internal (GtkWidget         *widget,
 
   info = gtk_drag_get_source_info (context, TRUE);
   
-  info->widget = gtk_widget_ref (widget);
+  info->widget = g_object_ref (widget);
   info->target_list = target_list;
   gtk_target_list_ref (target_list);
 
@@ -1764,10 +1771,21 @@ gtk_drag_source_info_destroy (GtkDragSourceInfo *info)
   g_free (info);
 }
 
+static gboolean
+drag_drop_finished_idle_cb (gpointer data)
+{
+  gtk_drag_source_info_destroy (data);
+  return FALSE;
+}
+
 static void
 gtk_drag_drop_finished (GtkDragSourceInfo *info)
 {
-  gtk_drag_source_info_destroy (info);
+  /* Workaround for the fact that the NS API blocks until the drag is
+   * over. This way the context is still valid when returning from
+   * drag_begin, even if it will still be quite useless. See bug #501588.
+  */
+  g_idle_add (drag_drop_finished_idle_cb, info);
 }
 
 /*************************************************************
