@@ -25,7 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include "gtkhandlebox.h"
 #include "gtkinvisible.h"
@@ -50,7 +50,8 @@ enum {
   PROP_SHADOW_TYPE,
   PROP_HANDLE_POSITION,
   PROP_SNAP_EDGE,
-  PROP_SNAP_EDGE_SET
+  PROP_SNAP_EDGE_SET,
+  PROP_CHILD_DETACHED
 };
 
 #define DRAG_HANDLE_SIZE 10
@@ -145,7 +146,7 @@ static void gtk_handle_box_end_drag       (GtkHandleBox      *hb,
 
 static guint        handle_box_signals[SIGNAL_LAST] = { 0 };
 
-G_DEFINE_TYPE (GtkHandleBox, gtk_handle_box, GTK_TYPE_BIN);
+G_DEFINE_TYPE (GtkHandleBox, gtk_handle_box, GTK_TYPE_BIN)
 
 static void
 gtk_handle_box_class_init (GtkHandleBoxClass *class)
@@ -168,7 +169,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    g_param_spec_enum ("shadow", NULL,
                                                       P_("Deprecated property, use shadow_type instead"),
 						      GTK_TYPE_SHADOW_TYPE,
-						      GTK_SHADOW_ETCHED_OUT,
+						      GTK_SHADOW_OUT,
                                                       GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW_TYPE,
@@ -176,7 +177,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                                       P_("Shadow type"),
                                                       P_("Appearance of the shadow that surrounds the container"),
 						      GTK_TYPE_SHADOW_TYPE,
-						      GTK_SHADOW_ETCHED_OUT,
+						      GTK_SHADOW_OUT,
                                                       GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
@@ -205,6 +206,14 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
 							 FALSE,
 							 GTK_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_CHILD_DETACHED,
+                                   g_param_spec_boolean ("child-detached",
+							 P_("Child Detached"),
+							 P_("A boolean value indicating whether the handlebox's child is attached or detached."),
+							 FALSE,
+							 GTK_PARAM_READABLE));
+
   object_class->destroy = gtk_handle_box_destroy;
 
   widget_class->map = gtk_handle_box_map;
@@ -323,6 +332,9 @@ gtk_handle_box_get_property (GObject         *object,
     case PROP_SNAP_EDGE_SET:
       g_value_set_boolean (value, handle_box->snap_edge != -1);
       break;
+    case PROP_CHILD_DETACHED:
+      g_value_set_boolean (value, handle_box->child_detached);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -441,7 +453,8 @@ gtk_handle_box_realize (GtkWidget *widget)
 			   GDK_LEAVE_NOTIFY_MASK |
 			   GDK_FOCUS_CHANGE_MASK |
 			   GDK_STRUCTURE_MASK);
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+  attributes.type_hint = GDK_WINDOW_TYPE_HINT_TOOLBAR;
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP | GDK_WA_TYPE_HINT;
   hb->float_window = gdk_window_new (gtk_widget_get_root_window (widget),
 				     &attributes, attributes_mask);
   gdk_window_set_user_data (hb->float_window, widget);
@@ -540,7 +553,7 @@ gtk_handle_box_size_request (GtkWidget      *widget,
     }
 
   /* if our child is not visible, we still request its size, since we
-   * won't have any usefull hint for our size otherwise.
+   * won't have any useful hint for our size otherwise.
    */
   if (bin->child)
     gtk_widget_size_request (bin->child, &child_requisition);
@@ -744,12 +757,12 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
 
 static void
 draw_textured_frame (GtkWidget *widget, GdkWindow *window, GdkRectangle *rect, GtkShadowType shadow,
-		     GdkRectangle *clip)
+		     GdkRectangle *clip, GtkOrientation orientation)
 {
    gtk_paint_handle (widget->style, window, GTK_STATE_NORMAL, shadow,
 		     clip, widget, "handlebox",
 		     rect->x, rect->y, rect->width, rect->height, 
-		     GTK_ORIENTATION_VERTICAL);
+		     orientation);
 }
 
 void
@@ -849,6 +862,24 @@ gtk_handle_box_get_snap_edge (GtkHandleBox *handle_box)
   return handle_box->snap_edge;
 }
 
+/**
+ * gtk_handle_box_get_child_detached:
+ * @handle_box: a #GtkHandleBox
+ *
+ * Whether the handlebox's child is currently detached.
+ *
+ * Return value: %TRUE if the child is currently detached, otherwise %FALSE
+ *
+ * Since: 2.14
+ **/
+gboolean
+gtk_handle_box_get_child_detached (GtkHandleBox *handle_box)
+{
+  g_return_val_if_fail (GTK_IS_HANDLE_BOX (handle_box), FALSE);
+
+  return handle_box->child_detached;
+}
+
 static void
 gtk_handle_box_paint (GtkWidget      *widget,
 		  
@@ -857,10 +888,11 @@ gtk_handle_box_paint (GtkWidget      *widget,
 {
   GtkBin *bin;
   GtkHandleBox *hb;
-  guint width, height;
+  gint width, height;
   GdkRectangle rect;
   GdkRectangle dest;
   gint handle_position;
+  GtkOrientation handle_orientation;
 
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
@@ -899,31 +931,39 @@ gtk_handle_box_paint (GtkWidget      *widget,
       rect.y = 0; 
       rect.width = DRAG_HANDLE_SIZE;
       rect.height = height;
+      handle_orientation = GTK_ORIENTATION_VERTICAL;
       break;
     case GTK_POS_RIGHT:
       rect.x = width - DRAG_HANDLE_SIZE; 
       rect.y = 0;
       rect.width = DRAG_HANDLE_SIZE;
       rect.height = height;
+      handle_orientation = GTK_ORIENTATION_VERTICAL;
       break;
     case GTK_POS_TOP:
       rect.x = 0;
       rect.y = 0; 
       rect.width = width;
       rect.height = DRAG_HANDLE_SIZE;
+      handle_orientation = GTK_ORIENTATION_HORIZONTAL;
       break;
     case GTK_POS_BOTTOM:
       rect.x = 0;
       rect.y = height - DRAG_HANDLE_SIZE;
       rect.width = width;
       rect.height = DRAG_HANDLE_SIZE;
+      handle_orientation = GTK_ORIENTATION_HORIZONTAL;
+      break;
+    default: 
+      g_assert_not_reached ();
       break;
     }
 
   if (gdk_rectangle_intersect (event ? &event->area : area, &rect, &dest))
     draw_textured_frame (widget, hb->bin_window, &rect,
 			 GTK_SHADOW_OUT,
-			 event ? &event->area : area);
+			 event ? &event->area : area,
+			 handle_orientation);
 
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
     (* GTK_WIDGET_CLASS (gtk_handle_box_parent_class)->expose_event) (widget, event);
