@@ -77,6 +77,7 @@ GtkStatusIcon        *gtk_status_icon_new_from_pixbuf    (GdkPixbuf          *pi
 GtkStatusIcon        *gtk_status_icon_new_from_file      (const gchar        *filename);
 GtkStatusIcon        *gtk_status_icon_new_from_stock     (const gchar        *stock_id);
 GtkStatusIcon        *gtk_status_icon_new_from_icon_name (const gchar        *icon_name);
+GtkStatusIcon        *gtk_status_icon_new_from_gicon     (GIcon              *icon);
 
 void                  gtk_status_icon_set_from_pixbuf    (GtkStatusIcon      *status_icon,
 							  GdkPixbuf          *pixbuf);
@@ -86,12 +87,16 @@ void                  gtk_status_icon_set_from_stock     (GtkStatusIcon      *st
 							  const gchar        *stock_id);
 void                  gtk_status_icon_set_from_icon_name (GtkStatusIcon      *status_icon,
 							  const gchar        *icon_name);
+void                  gtk_status_icon_set_from_gicon     (GtkStatusIcon      *status_icon,
+                                                          GIcon              *icon);
 
 GtkImageType          gtk_status_icon_get_storage_type   (GtkStatusIcon      *status_icon);
 
 GdkPixbuf            *gtk_status_icon_get_pixbuf         (GtkStatusIcon      *status_icon);
 G_CONST_RETURN gchar *gtk_status_icon_get_stock          (GtkStatusIcon      *status_icon);
 G_CONST_RETURN gchar *gtk_status_icon_get_icon_name      (GtkStatusIcon      *status_icon);
+void                  gtk_status_icon_get_gicon          (GtkStatusIcon      *status_icon,
+                                                          GIcon             **icon);
 
 gint                  gtk_status_icon_get_size           (GtkStatusIcon      *status_icon);
 
