@@ -17,7 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
@@ -389,7 +389,7 @@ GtkTreePath *
 gtk_tree_path_new (void)
 {
   GtkTreePath *retval;
-  retval = g_new (GtkTreePath, 1);
+  retval = g_slice_new (GtkTreePath);
   retval->depth = 0;
   retval->indices = NULL;
 
@@ -632,7 +632,7 @@ gtk_tree_path_free (GtkTreePath *path)
     return;
 
   g_free (path->indices);
-  g_free (path);
+  g_slice_free (GtkTreePath, path);
 }
 
 /**
@@ -650,7 +650,7 @@ gtk_tree_path_copy (const GtkTreePath *path)
 
   g_return_val_if_fail (path != NULL, NULL);
 
-  retval = g_new (GtkTreePath, 1);
+  retval = g_slice_new (GtkTreePath);
   retval->depth = path->depth;
   retval->indices = g_new (gint, path->depth);
   memcpy (retval->indices, path->indices, path->depth * sizeof (gint));
@@ -850,10 +850,11 @@ gtk_tree_path_down (GtkTreePath *path)
  * gtk_tree_iter_copy:
  * @iter: A #GtkTreeIter.
  *
- * Creates a dynamically allocated tree iterator as a copy of @iter.  This
- * function is not intended for use in applications, because you can just copy
- * the structs by value (<literal>GtkTreeIter new_iter = iter;</literal>).  You
- * must free this iter with gtk_tree_iter_free ().
+ * Creates a dynamically allocated tree iterator as a copy of @iter.  
+ * This function is not intended for use in applications, because you 
+ * can just copy the structs by value 
+ * (<literal>GtkTreeIter new_iter = iter;</literal>).
+ * You must free this iter with gtk_tree_iter_free().
  *
  * Return value: a newly-allocated copy of @iter.
  **/
@@ -874,8 +875,8 @@ gtk_tree_iter_copy (GtkTreeIter *iter)
  * gtk_tree_iter_free:
  * @iter: A dynamically allocated tree iterator.
  *
- * Frees an iterator that has been allocated on the heap.  This function is
- * mainly used for language bindings.
+ * Frees an iterator that has been allocated by gtk_tree_iter_copy().
+ * This function is mainly used for language bindings.
  **/
 void
 gtk_tree_iter_free (GtkTreeIter *iter)
