@@ -26,12 +26,13 @@
 
 #undef GTK_DISABLE_DEPRECATED
 
-#include <config.h>
+#include "config.h"
 #include <string.h> /* strcmp */
 
-#include "gtkalias.h"
 #include "gtktypeutils.h"
 #include "gtkobject.h"
+#include "gtkintl.h"
+#include "gtkalias.h"
 
 
 /* --- functions --- */
@@ -59,6 +60,18 @@ gtk_type_unique (GtkType            parent_type,
   return g_type_register_static (parent_type, gtkinfo->type_name, &tinfo, 0);
 }
 
+/**
+ * gtk_type_class
+ * @type: a #GtkType.
+ *
+ * Returns a pointer pointing to the class of @type or %NULL if there
+ * was any trouble identifying @type.  Initializes the class if
+ * necessary.
+ *
+ * Returns: pointer to the class.
+ *
+ * Deprecated: 2.14: Use g_type_class_peek() or g_type_class_ref() instead.
+ **/
 gpointer
 gtk_type_class (GtkType type)
 {
@@ -132,7 +145,7 @@ gtk_identifier_get_type (void)
   if (our_type == 0)
     {
       GTypeInfo tinfo = { 0, };
-      our_type = g_type_register_static (G_TYPE_STRING, "GtkIdentifier", &tinfo, 0);
+      our_type = g_type_register_static (G_TYPE_STRING, I_("GtkIdentifier"), &tinfo, 0);
     }
 
   return our_type;
@@ -198,3 +211,6 @@ gtk_type_flags_find_value (GtkType      flags_type,
   return value;
 }
 
+
+#define __GTK_TYPE_UTILS_C__
+#include "gtkaliasdef.c"
